#include<stdio.h>

#include<cv.h>
#include<cxcore.h>
#include<highgui.h>

class histogram {
public:
    histogram(int b) { bins = b; h = new int[b]; assert (h != 0); norm = 0; }
    ~histogram() { if (h) delete[] h; }
    int bins;
    int *h;
    int norm;
    int max;
};

bool createHistogram(IplImage *src, histogram *histo) {

    assert (src->nChannels == 1);
    unsigned char *data = (unsigned char *)src->imageData;
    memset (histo->h, 0, sizeof(int) * histo->bins);
    for (int i = 0; i < src->height; i++) {
        for (int j = 0; j < src->width; j++) {
            histo->h[*data] ++;
            data++;
        }
        data += (src->widthStep-src->width);
    }

    histo->norm = src->width*src->height;
    histo->max = 0;
    for (int k = 0; k < histo->bins; k++) {
        if (histo->h[k] > histo->max) histo->max = histo->h[k];
    }
    return true;
}


bool createImageFromHistogram(histogram *histo, IplImage *dest) {

    memset (dest->imageData, 0, dest->imageSize);
    unsigned char *data = (unsigned char *)dest->imageData;
    for (int k = 0; k < histo->bins; k++) {
        int row = 255 - int(double(histo->h[k])/double(histo->max) * 255 + .5);
        *(data + row * dest->widthStep + k) = 255;
    }
    return true;
}


bool createEqualizeLUT(histogram *histo, histogram *lut) {

    assert (histo->bins == lut->bins);
    memset (lut->h, 0, sizeof(int) * lut->bins);
    double *temp = new double[lut->bins];
    assert (temp != 0);

    temp[0] = double(histo->h[0])/double(histo->norm);
    for (int k = 1; k < histo->bins; k++) {
        temp[k] = temp[k-1] + double(histo->h[k]) / double(histo->norm);
    }

    double max = temp[0];
    double min = temp[0];
    for (int k = 1; k < lut->bins; k++) {
        if (temp[k] > max) 
            max = temp[k];
        else 
        if (temp[k] < min) 
            min = temp[k];
    }
   
    for (int k = 0; k < lut->bins; k++) {
        lut->h[k] = int ((temp[k] - min) / (max-min) * 255 + .5);
    }

    lut->max = 256;
    lut->norm = 256;
    return true;
}


bool equalize(IplImage *img, histogram *lut) {

    unsigned char *data = (unsigned char *)img->imageData;
    for (int i = 0; i < img->height; i++) {
        for (int j = 0; j < img->width; j++) {
            *data = lut->h[*data];
            data++;
        }

        data += (img->widthStep - img->width);
    }
    return true;
}


int main(int argc, char **argv) {
    
    printf("hello opencv!\n");
    
    IplImage *first;
    first = cvLoadImage("lena.bmp", CV_LOAD_IMAGE_GRAYSCALE);
    if (first == 0) {
        fprintf(stderr, "Error: cannot load first image\n");
        return -1;
    }
    
    cvNamedWindow("First");
    cvShowImage("First", first);
    cvWaitKey(0);

    CvSize size;
    size.height = 256;
    size.width = 256;
    IplImage *second = cvCreateImage(size, IPL_DEPTH_8U, 1);
    assert (second != 0);

    /* build histogram */
    histogram histo(256);
    createHistogram(first, &histo);
    createImageFromHistogram(&histo, second);

    cvNamedWindow("Second");
    cvShowImage("Second", second);
    cvWaitKey(0);
    
    /* equalize */
    histogram lut(256);
    createEqualizeLUT(&histo, &lut);
    createImageFromHistogram(&lut, second);
    cvShowImage("Second", second);
    cvWaitKey(0);
    
    equalize(first, &lut);
    cvShowImage("First", first);
    cvWaitKey(0);

    createHistogram(first, &histo);
    createImageFromHistogram(&histo, second);

    cvNamedWindow("Second");
    cvShowImage("Second", second);
    cvWaitKey(0);

    cvReleaseImage(&first);
    cvDestroyWindow("First");
    cvReleaseImage(&second);
    cvDestroyWindow("Second");
    return 0;
}
